@@ -20,6 +20,13 @@
 
 #include "atkcomponent.h"
 
+enum {
+  BOUNDS_CHANGED,
+  LAST_SIGNAL
+};
+
+static void       atk_component_base_init (AtkComponentIface *class);
+
 static gboolean   atk_component_real_contains                (AtkComponent *component,
                                                               gint         x,
                                                               gint         y,
@@ -39,8 +46,10 @@ static void      atk_component_real_get_size                 (AtkComponent *comp
                                                               gint         *width,
                                                               gint         *height);
 
+static guint atk_component_signals[LAST_SIGNAL] = { 0 };
+
 GType
-atk_component_get_type ()
+atk_component_get_type (void)
 {
   static GType type = 0;
 
@@ -48,7 +57,7 @@ atk_component_get_type ()
     static const GTypeInfo tinfo =
     {
       sizeof (AtkComponentIface),
-      (GBaseInitFunc) NULL,
+      (GBaseInitFunc) atk_component_base_init,
       (GBaseFinalizeFunc) NULL,
 
     };
@@ -59,15 +68,44 @@ atk_component_get_type ()
   return type;
 }
 
+static void
+atk_component_base_init (AtkComponentIface *class)
+{
+  static gboolean initialized = FALSE;
+
+  if (! initialized)
+    {
+      class->ref_accessible_at_point = atk_component_real_ref_accessible_at_point;
+      class->contains = atk_component_real_contains;
+      class->get_position = atk_component_real_get_position;
+      class->get_size = atk_component_real_get_size;
+
+      atk_component_signals[BOUNDS_CHANGED] =
+        g_signal_new ("bounds_changed",
+                      ATK_TYPE_COMPONENT,
+                      G_SIGNAL_RUN_LAST,
+                      G_STRUCT_OFFSET (AtkComponentIface, bounds_changed),
+                      (GSignalAccumulator) NULL, NULL,
+                      g_cclosure_marshal_VOID__BOXED,
+                      G_TYPE_NONE, 1,
+                      ATK_TYPE_RECTANGLE | G_SIGNAL_TYPE_STATIC_SCOPE);
+
+      initialized = TRUE;
+    }
+}
+
+
 /**
  * atk_component_add_focus_handler:
  * @component: The #AtkComponent to attach the @handler to
  * @handler: The #AtkFocusHandler to be attached to @component
  *
  * Add the specified handler to the set of functions to be called 
- * when this object receives focus events (in or out).
+ * when this object receives focus events (in or out). If the handler is
+ * already added it is not added again
  *
- * Returns:
+ * Returns: a handler id which can be used in atk_component_remove_focus_handler
+ * or zero if the handler was already added.
  **/
 guint
 atk_component_add_focus_handler (AtkComponent    *component,
@@ -134,12 +172,7 @@ atk_component_contains (AtkComponent    *component,
   if (iface->contains)
     return (iface->contains) (component, x, y, coord_type);
   else
-  {
-    /*
-     * if this method is not overridden use the default implementation.
-     */
-    return atk_component_real_contains (component, x, y, coord_type);
-  }
+    return FALSE;
 }
 
 /**
@@ -169,12 +202,7 @@ atk_component_ref_accessible_at_point (AtkComponent    *component,
   if (iface->ref_accessible_at_point)
     return (iface->ref_accessible_at_point) (component, x, y, coord_type);
   else
-  {
-    /*
-     * if this method is not overridden use the default implementation.
-     */
-    return atk_component_real_ref_accessible_at_point (component, x, y, coord_type);
-  }
+    return NULL;
 }
 
 /**
@@ -263,13 +291,6 @@ atk_component_get_position   (AtkComponent    *component,
 
   if (iface->get_position)
     (iface->get_position) (component, real_x, real_y, coord_type);
-  else
-  {
-    /*
-     * if this method is not overridden use the default implementation.
-     */
-    atk_component_real_get_position (component, real_x, real_y, coord_type);
-  }
 }
 
 /**
@@ -300,20 +321,59 @@ atk_component_get_size       (AtkComponent    *component,
   else
     real_height = &local_height;
 
-  iface = ATK_COMPONENT_GET_IFACE (component);
   g_return_if_fail (ATK_IS_COMPONENT (component));
 
   iface = ATK_COMPONENT_GET_IFACE (component);
 
   if (iface->get_size)
     (iface->get_size) (component, real_width, real_height);
+}
+
+/**
+ * atk_component_get_layer:
+ * @component: an #AtkComponent
+ *
+ * Gets the layer of the component.
+ *
+ * Returns: an #AtkLayer which is the layer of the component
+ **/
+AtkLayer
+atk_component_get_layer (AtkComponent *component) 
+{
+  AtkComponentIface *iface;
+
+  g_return_val_if_fail (ATK_IS_COMPONENT (component), ATK_LAYER_INVALID);
+
+  iface = ATK_COMPONENT_GET_IFACE (component);
+  if (iface->get_layer)
+    return (iface->get_layer) (component);
   else
-  {
-    /*
-     * if this method is not overridden use the default implementation.
-     */
-    atk_component_real_get_size (component, real_width, real_height);
-  }
+    return ATK_LAYER_WIDGET;
+}
+
+/**
+ * atk_component_get_mdi_zorder:
+ * @component: an #AtkComponent
+ *
+ * Gets the zorder of the component. The value G_MININT will be returned 
+ * if the layer of the component is not ATK_LAYER_MDI or ATK_LAYER_WINDOW.
+ *
+ * Returns: a gint which is the zorder of the component, i.e. the depth at 
+ * which the component is shown in relation to other components in the same 
+ * container.
+ **/
+gint
+atk_component_get_mdi_zorder (AtkComponent *component) 
+{
+  AtkComponentIface *iface;
+
+  g_return_val_if_fail (ATK_IS_COMPONENT (component), G_MININT);
+
+  iface = ATK_COMPONENT_GET_IFACE (component);
+  if (iface->get_mdi_zorder)
+    return (iface->get_mdi_zorder) (component);
+  else
+    return G_MININT;
 }
 
 /**
@@ -504,3 +564,25 @@ atk_component_real_get_size (AtkComponent *component,
 
   atk_component_get_extents (component, &x, &y, width, height, coord_type);
 }
+
+static AtkRectangle *
+atk_rectangle_copy (const AtkRectangle *rectangle)
+{
+  AtkRectangle *result = g_new (AtkRectangle, 1);
+  *result = *rectangle;
+
+  return result;
+}
+
+GType
+atk_rectangle_get_type (void)
+{
+  static GType our_type = 0;
+
+  if (our_type == 0)
+    our_type = g_boxed_type_register_static ("AtkRectangle",
+                                             (GBoxedCopyFunc)atk_rectangle_copy,
+                                             (GBoxedFreeFunc)g_free);
+  return our_type;
+}
+
